#include "stdafx.h"
#include "TaskSimulate.h"

const int MAX_TRACES=100;
const int MAX_POINTS=10000000;

const int SPINLOW=0;
const int SPINMID=10;
const int SPINHIGH=20;

const COLORREF CR_LISTITEM = RGB(192,255,192);	// highlight background for lists 
const COLORREF CR_LISTHIGHLIGHT = RGB(64,255,0);	// highlight Current list item 
const COLORREF CR_HIGHLIGHT = RGB(255,255,128);	// highlight background for current selection
const COLORREF CR_DRAGGING = RGB(255,255,0);
const COLORREF CR_BGR1 = RGB(255,255,255);		// alternating traces backgrounds
const COLORREF CR_BGR2 = RGB(240,240,240);		// 

const char * g_pchDataFilter = "Data (*.qdf)\0*.qdf\0All Files (*.*)\0*.*\0\0";
const char * g_pchDataExtension = "qdf";

const int MAXSTATUSTEXTLEN = 256;
const int MAXTIPTEXTLEN = 6096;


//-------------------------------------------------------------------------
class CVwData : public CDocumentDialog { 
	public :
		CData m_oData;			//-- Data file object - memory representation of data 
		

		int m_iCurrentTrace;	//-- non-visible traces prior to first visible point
		int m_iDrawSeg;			//-- segment which contains first visible point
		int m_iDrawOffset;		//-- offset within DrawSeg to first visible point

		//-- display area 
		HBITMAP m_hBitmap;		// Handle to dc and bitmap for display 
		HDC m_hdcBitmap;
		RECT m_rctData;			// coordinates of display area 
		bool m_bSetupBitmap;	// View bitmap has been created 
		bool m_bDataDrawn;		// data has been drawn to bitmap

		//-- drag support for highlighting data sections
		int m_iXHit, m_iYHit;
		int m_iXDrag, m_iYDrag;
		bool m_bDrag;			// true if this data window has mouse captured for selecting data 
		
		//-- support for a 'current' selection of data 
		int m_iSelStartSeg, m_iSelStopSeg, m_iSelStartOffset, m_iSelStopOffset;
		bool m_bSelect;			// true if data selection has been made

		//-- Dialogue control handles 
		CDlgCombo m_cmbList;
		CDlgButton m_btnAddList;
		CDlgButton m_btnDelList;
		CDlgButton m_btnAddAllSegments;
		CDlgCombo m_cmbListItems;	
		CDlgButton m_btnAddListItem;
		CDlgButton m_btnDelListItem;
		CDlgButton m_btnListItemInfo;

		CDlgButton m_btnIdl;
		CDlgButton m_btnConfig;
		CDlgControl m_picData;
		CDlgScroll m_scrData;
		CDlgControl m_spinPoints, m_spinTraces;

		CDlgButton m_btnStatusCopy;
		CDlgCheck m_chkStatus;
		char m_bufTip[MAXTIPTEXTLEN];

		//-- constructor
		CVwData() : CDocumentDialog( g_pchDataFilter, g_pchDataExtension ) { 
			m_bufTip[0]=0;
			m_hBitmap = NULL;
			m_bSetupBitmap=false;
			m_bDataDrawn=false;

			m_iID = IDD_DATAVIEW;
			m_pCapture=NULL;	// control when mouse is captured 

			m_iDrawSeg=0;
			m_iCurrentTrace=0;
			m_iDrawOffset=0;
			
			m_bDrag=false;
			m_iXDrag=m_iXHit=m_iYDrag=m_iYHit=0;
			m_bSelect=false;
			m_iSelStartSeg=m_iSelStopSeg=m_iSelStartOffset=m_iSelStopOffset=0;
			}
			
		//-- destructor
		~CVwData() { 
			if( m_hBitmap != NULL ) {
				DeleteObject( m_hBitmap );
				DeleteDC( m_hdcBitmap );
				}
			}

		//-- Initialize Controls - set handles, etc.
		void InitializeControls( ) { 
			m_cmbList.SetHandle( GetDlgItem( m_hDlg, IDC_CMBLIST), "Selection list names" );
			m_cmbList.ClearContents();
			m_btnAddList.SetHandle( GetDlgItem( m_hDlg, IDC_BTNADDLIST ), "Create a new selection list" );
			m_btnDelList.SetHandle( GetDlgItem( m_hDlg, IDC_BTNDELLIST ), "Delete the current selection list" );
			m_btnAddAllSegments.SetHandle( GetDlgItem( m_hDlg, IDC_BTNADDALLSEGMENTS ), "Add a new list with an item for each data segment." );

			m_cmbListItems.SetHandle( GetDlgItem( m_hDlg, IDC_CMBLISTITEM), "Selection list contents" );
			m_cmbListItems.ClearContents();
			m_btnAddListItem.SetHandle( GetDlgItem( m_hDlg, IDC_BTNADDLISTITEM ), "Append currently selected data to the currently selected list." );
			m_btnDelListItem.SetHandle( GetDlgItem( m_hDlg, IDC_BTNDELLISTITEM ), "Delete the current list entry." );
			m_btnListItemInfo.SetHandle( GetDlgItem( m_hDlg, IDC_BTNLISTITEMINFO ), "View/edit list item label and note." );
			
			
			m_btnIdl.SetHandle( GetDlgItem( m_hDlg, IDC_BTNIDL), "Create New Idealization" );
			m_btnConfig.SetHandle( GetDlgItem( m_hDlg, IDC_BTNCONFIG), "Data view configuration" );
			m_picData.SetHandle( GetDlgItem( m_hDlg, IDC_DATA), "Data display area" );
			m_scrData.SetHandle( GetDlgItem( m_hDlg, IDC_SCROLLDATA), "Scroll through data file" );
			m_scrData.SetRange( 0, 100, 2, 10 );
			m_spinPoints.SetHandle( GetDlgItem( m_hDlg, IDC_SPINPOINTS), "Change the number of points displayed per trace" );
			SendMessage( m_spinPoints.GetHandle(), UDM_SETRANGE, 0, MAKELONG( SPINHIGH, SPINLOW ) );
			SendMessage( m_spinPoints.GetHandle(), UDM_SETPOS, 0, MAKELONG( SPINMID, 0 ) );
			m_spinTraces.SetHandle( GetDlgItem( m_hDlg, IDC_SPINTRACES), "Change the number of traces displayed" );
			SendMessage( m_spinTraces.GetHandle(), UDM_SETRANGE, 0, MAKELONG( SPINHIGH, SPINLOW ) );
			SendMessage( m_spinTraces.GetHandle(), UDM_SETPOS, 0, MAKELONG( SPINMID, 0 ) );
			
			m_chkStatus.SetHandle( GetDlgItem( m_hDlg, IDC_CHK_STATUS), "Display statistics for current selection" );
			m_chkStatus.SetCheck(true);

			m_btnStatusCopy.SetHandle( GetDlgItem( m_hDlg, IDC_BTNSTATUSCOPY), "Copy status text to clipboard" );
			m_btnStatusCopy.SetButtonImage( GetInstance(), IDI_CLIPCOPY, 16, 16 );			
			}

		void FillListControl( int iSetCurrList=-1 ) { 
			//-- repopulate lists 
			m_cmbList.ClearContents();
			for( int i=0; i<m_oData.m_aLists.Count(); ++i ) {
				CSelList * pList = m_oData.List(i);
				m_cmbList.AddString( pList->m_cName.Buffer() );
				}
			if( iSetCurrList >=0 ) 
				m_cmbList.SetCurSel( iSetCurrList );
			}
			
		void FillListItemControl( int iSetCurrListItem=-1 ) { 
			m_cmbListItems.ClearContents();
			int iList = m_cmbList.GetCurSel();
			if( iList<0 ) 
				return;
				
			//-- Repopulate list items
			char buf[256];
			CSelList * pList = m_oData.List(iList);
			for( int iItem=0; iItem<pList->m_aItems.Count(); ++iItem ) { 
				CSelListItem * pItem = pList->Item(iItem);
				pItem->Text(buf);
				m_cmbListItems.AddString( buf );
				if( iSetCurrListItem==iItem ) 
					m_cmbListItems.SetCurSel(iItem);
				}
			}

		void * GetData( int iData, void * pInfo ) { 
			if( iData == 1 ) 
				return m_cFileName.ConstBuffer();
			if( iData == 2 ) 
				return &m_oData;
			if( iData == 3 ) { 
				((int *)pInfo)[0]=m_iSelStartSeg;
				((int *)pInfo)[1]=m_iSelStartOffset;
				((int *)pInfo)[2]=m_iSelStopSeg;
				((int *)pInfo)[3]=m_iSelStopOffset;
				return NULL;
				}
			if( iData == 4 ) {
				*(int *)pInfo=m_cmbList.GetCurSel();
				return NULL;
				}
				
			return NULL;
			}


		//------------------------------------------
		//Control Message Handlers

		//-- update text in status bar
		void SetStatusText() { 
			char bufStatus[MAXSTATUSTEXTLEN];
			
			//-- if status display is off, exit.
			if( ! m_chkStatus.GetCheck() )
				return;
				
			//-- init status text to selected point range
			sprintf( bufStatus, "[%d:%d-%d:%d] ", 
					m_iSelStartSeg, m_iSelStartOffset, m_iSelStopSeg, m_iSelStopOffset);
				
			//-- Calculate average and std.dev. for each channel	 
			char bufChan[256];
			int iPoints, iCh, iChs=m_oData.m_oConfig.ChannelCount();
			double * adAvg = new double[iChs];
			double * adStd = new double[iChs];
			
			m_oData.CalculateStatistics( 0, iChs-1, 
										m_iSelStartSeg, m_iSelStartOffset, m_iSelStopSeg, m_iSelStopOffset, 
										&iPoints, adAvg, adStd );
			double dTm = m_oData.m_dSampling * m_iSelStartOffset;
										
			//-- Build Status message
			sprintf( bufStatus+strlen(bufStatus), "[%d points @ %.3f] [Avg:", iPoints, dTm );
			for( iCh=0; iCh<iChs; ++iCh ) {
				sprintf( bufChan, iCh==iChs-1?"%f]":"%f,", adAvg[iCh] );				 
				if( strlen(bufChan)+strlen(bufStatus)+10>MAXSTATUSTEXTLEN ) 
					break;
				strcat( bufStatus, bufChan );
				}
			sprintf( bufStatus+strlen(bufStatus), "[Std:" );
			for( iCh=0; iCh<iChs; ++iCh ) {
				sprintf( bufChan, iCh==iChs-1?"%f]":"%f,", adStd[iCh] );				 
				if( strlen(bufChan)+strlen(bufStatus)>MAXSTATUSTEXTLEN ) 
					break;
				strcat( bufStatus, bufChan );
				}

			//-- build tip message 
			sprintf( m_bufTip, "%d:%d-%d:%d  %d points", m_iSelStartSeg, m_iSelStartOffset, m_iSelStopSeg, m_iSelStopOffset, iPoints);
			for( iCh=0; iCh<iChs; ++iCh ) {
				if( strlen(m_bufTip)+128<MAXTIPTEXTLEN ) 
					sprintf( m_bufTip+strlen(m_bufTip), "\nCh %d\t%.2f\t%.2f", 
									iCh, adAvg[iCh], adStd[iCh] );				 
				}
				
			//-- update control with new status string
			m_chkStatus.SetText( bufStatus );
			m_chkStatus.CreateTip( m_bufTip, 500 );

			//-- Cleanup 
			delete [] adAvg;
			delete [] adStd;
			}
			
		void CopyStatusToClipboard() { 
			if( OpenClipboard( m_hDlg ) ) { 
				EmptyClipboard();

				//-- Copy text to a global memory object then copy to clipboard
				HGLOBAL hGl = GlobalAlloc(GMEM_MOVEABLE, 1+strlen(m_bufTip) );
				if( hGl!=NULL ) { 
					char * pLock = (char *)GlobalLock(hGl); 
					strcpy( pLock, m_bufTip );
			        GlobalUnlock(hGl); 
			        SetClipboardData(CF_TEXT, hGl);
			        }
				CloseClipboard( );
				}
			}
			
		//-- Toggle status bar display 
		void OnCheckStatus() { 
			if( m_chkStatus.GetCheck() )
				SetStatusText();
			else
				m_chkStatus.SetText("Status");
			}

		//-- Check if user wants to save file before closing window.  false if user cancels save.		
		bool Close( ) { 
			bool bResult = ( ! Updated() || OnSave() ); 
			m_oData.ClearUpdated();
			return bResult;
			}	

		//----- Check if a window can be closed.
		bool CanClose( ) { 
			return m_oData.GetStatus()==0; 
			}	

		//-- Override virtual Updated() function.
		bool Updated() { 
			return m_oData.m_bDataUpdated 
				|| m_oData.m_bListsUpdated 
				|| m_oData.m_bCursorsUpdated 
				|| m_oData.m_bFitCurvesUpdated 
				|| m_oData.m_bIdlUpdated
				|| m_oData.m_bConfigUpdated;
			}

		//-- Sent when dialogue exit occurs 
		bool OnSave() { 
			if( ! GetSaveAsFileName( ) )
				return false;
			m_oData.WriteQDF( m_cFileName.ConstBuffer() );
			UpdateDataMRU( m_cFileName.ConstBuffer() );
			return true;
			}

		void OnExit() { 
			if( Updated() ) 
				OnSave();
			}

		//-- open configure view dialogue
		void OnBtnConfig() { 
			if( OpenDataViewConfig( m_hDlg, &m_oData.m_oConfig ) )  {
				OnDataChanged();
				m_oData.SetConfigUpdated();
				}
			}

		//-- Convert screen x,y coordinate to data segment, offset values 
		// optionally enforce calculation to lie on actual data point, not blank space
		void PointToSegmentOffset( int iX, int iY, int * piSeg, int * piOffset ) {
			int iHgt=(m_rctData.bottom-m_rctData.top-1)/m_oData.m_oConfig.m_iTracesPerPage;
			int iWid=(m_rctData.right-m_rctData.left);

			//-- index to first trace 
			int iSeg = m_iDrawSeg;
			int iOffset = m_iDrawOffset;
			CSegment * pSeg = m_oData.Segment(iSeg);

			//-- iterate through traces
			for( int iTrace=0; iTrace<m_oData.m_oConfig.m_iTracesPerPage; ++iTrace ) { 
				if( iY-m_rctData.top<=iHgt*(iTrace+1)-1  ) {	//-- is point is in this trace ? 
					//-- Set results and return.
					*piSeg=iSeg;
					*piOffset=min( pSeg->m_iSamples-1, iOffset + max(0, int( (iX-m_rctData.left)*m_oData.m_oConfig.m_iPointsPerTrace/iWid)) );
					return;
					}
										
				//-- advance pointers
				iOffset+=m_oData.m_oConfig.m_iPointsPerTrace;
				if( iOffset >= pSeg->m_iSamples ) { 
					iOffset=0;
					++iSeg;
					if( iSeg>=m_oData.m_aSegments.Count() ) {
						// past end, return last point
						*piSeg=m_oData.m_aSegments.Count()-1;
						*piOffset=m_oData.Segment(*piSeg)->m_iSamples-1;
						return;
						}
					pSeg = m_oData.Segment(iSeg);
					}
				}
			}
		
		//-- Paint - ( Call background and foreground display funcs ) ( thread this ) 
		void DrawData() { 
			m_bDataDrawn=true;

			RECT rct, rctBase;
			HBRUSH brFill[2] = { CreateSolidBrush( CR_BGR1 ), CreateSolidBrush( CR_BGR2 ) };	
			HBRUSH brHighlight = CreateSolidBrush( CR_HIGHLIGHT );	
			HBRUSH brListItem = CreateSolidBrush( CR_LISTITEM );	
			HBRUSH brListHighlight = CreateSolidBrush( CR_LISTHIGHLIGHT );	
			HBRUSH brDragging = CreateSolidBrush( CR_DRAGGING  );	
			HPEN penRed = CreatePen( 0, 0, RGB(255,0,0) );
			HPEN penWas = (HPEN) SelectObject( m_hdcBitmap, penRed );

			//-- erase entire background
			int iHgt=(m_rctData.bottom-m_rctData.top-1)/m_oData.m_oConfig.m_iTracesPerPage;
			int iWid=(m_rctData.right-m_rctData.left);

			SetRect( &rct, 0, 0, iWid-1, m_rctData.bottom );
			FillRect( m_hdcBitmap, &rct, (HBRUSH)GetStockObject( WHITE_BRUSH ) );
			
			//-- setup first segment to display 
			CSegment * pSeg = NULL;
			CIdlList * pIdl = NULL; 
			int iSeg = m_iDrawSeg;
			int iOffset = m_iDrawOffset;
			char ach[16];

			//-- Keep drawing traces until all segments or all traces are completed
			double dPixPerSamp = double(iWid)/m_oData.m_oConfig.m_iPointsPerTrace;
			for( int iTrace=0; iTrace<m_oData.m_oConfig.m_iTracesPerPage; ++iTrace ) { 
				//-- Draw background for 1 panel (in alternating colors) 
				SetRect( &rctBase, 0, iHgt*iTrace, iWid-1, iHgt*(iTrace+1)-1 );
				rct=rctBase;
				FillRect( m_hdcBitmap, &rct, brFill[(m_iCurrentTrace+iTrace)%2] );
				
				//-- Highlight List areas
				int iList = m_cmbList.GetCurSel();
				if( iList>= 0 && ( m_oData.m_oConfig.m_bDispListLabels || m_oData.m_oConfig.m_bDispLists ) ) { 
					int iCurrItem = m_cmbListItems.GetCurSel();
					CSelList * pList = m_oData.List(iList);
					for( int iItem=0; iItem<pList->m_aItems.Count(); ++iItem ) { 
						CSelListItem * pItem = pList->Item(iItem);
						if( ( pItem->m_iStartSegment<iSeg || (pItem->m_iStartSegment==iSeg && pItem->m_iStartPoint<iOffset+m_oData.m_oConfig.m_iPointsPerTrace ) )
							&& ( pItem->m_iEndSegment>iSeg || (pItem->m_iEndSegment==iSeg && pItem->m_iEndPoint>iOffset ) )
							) {			// if this list seg is visible on trace 

							//-- set left edge of selection
							if( pItem->m_iStartSegment<iSeg || (pItem->m_iStartSegment==iSeg && pItem->m_iStartPoint<iOffset) )						
								rct.left = 0;
							else 
								rct.left = MulDiv(pItem->m_iStartPoint-iOffset, iWid, m_oData.m_oConfig.m_iPointsPerTrace );
							//-- set right edge of selection 
							if( pItem->m_iEndSegment>iSeg || (pItem->m_iEndSegment==iSeg && pItem->m_iEndPoint>iOffset+m_oData.m_oConfig.m_iPointsPerTrace ) )						
								rct.right = iWid;
							else 
								rct.right = MulDiv(pItem->m_iEndPoint-iOffset, iWid, m_oData.m_oConfig.m_iPointsPerTrace )+1;
	
							//-- draw higlight background 
							if( m_oData.m_oConfig.m_bDispLists ) 
								FillRect( m_hdcBitmap, &rct, iItem==iCurrItem ? brListHighlight : brListItem );

							//-- Draw list item label 
							if( m_oData.m_oConfig.m_bDispListLabels && pItem->m_cLabel.Length()>0 ) {
								COLORREF crWas = SetBkColor( m_hdcBitmap, iItem==iCurrItem ? CR_LISTHIGHLIGHT : CR_LISTITEM );
								DrawText( m_hdcBitmap, pItem->m_cLabel.ConstBuffer(), pItem->m_cLabel.Length(), &rct, 0 );
								SetBkColor( m_hdcBitmap, crWas );
								}

							}
						}
					}
				
				//-- Selection highlight leaves margin at top / bottom so overlapped list is visible.
				rct=rctBase;
				rct.top = min( rct.top+2, rct.bottom );
				rct.bottom = max( rct.top, rct.bottom-2 );
				
				//-- Highlight current selection 
				if( m_bSelect 
						&& ( m_iSelStartSeg<iSeg || (m_iSelStartSeg==iSeg && m_iSelStartOffset<iOffset+m_oData.m_oConfig.m_iPointsPerTrace ) )
						&& ( m_iSelStopSeg>iSeg || (m_iSelStopSeg==iSeg && m_iSelStopOffset>iOffset ) )
						) { 
					//-- set left edge of selection
					if( m_iSelStartSeg<iSeg || (m_iSelStartSeg==iSeg && m_iSelStartOffset<iOffset) )						
						rct.left = 0;
					else 
						rct.left = MulDiv(m_iSelStartOffset-iOffset, iWid, m_oData.m_oConfig.m_iPointsPerTrace );
					//-- set right edge of selection 
					if( m_iSelStopSeg>iSeg || (m_iSelStopSeg==iSeg && m_iSelStopOffset>iOffset+m_oData.m_oConfig.m_iPointsPerTrace ) )						
						rct.right = iWid;
					else { 
						if( iWid<=m_oData.m_oConfig.m_iPointsPerTrace ) 
							rct.right = MulDiv(m_iSelStopOffset-iOffset, iWid, m_oData.m_oConfig.m_iPointsPerTrace );
						else 
							rct.right = MulDiv(m_iSelStopOffset+1-iOffset, iWid, m_oData.m_oConfig.m_iPointsPerTrace )-1;
						}
						
					//-- draw higlight background 
					FillRect( m_hdcBitmap, &rct, brHighlight );
					}
				
				//-- Highlight dragging area
				if( m_bDrag ) { 
					rct=rctBase;
					if( m_iYHit-m_rctData.top<=rct.bottom && m_iYDrag-m_rctData.top>=rct.top ) {
						rct.left = m_iYHit-m_rctData.top<rct.top?rct.left:m_iXHit-m_rctData.left;
						rct.right = m_iYDrag-m_rctData.top>rct.bottom?rct.right:m_iXDrag-m_rctData.left;
						FillRect( m_hdcBitmap, &rct, brDragging );
						}
					else if( m_iYHit-m_rctData.top>=rct.top && m_iYDrag-m_rctData.top<=rct.bottom ) {
						rct.left = m_iYDrag-m_rctData.top<rct.top?rct.left:m_iXDrag-m_rctData.left;
						rct.right = m_iYHit-m_rctData.top>rct.bottom?rct.right:m_iXHit-m_rctData.left;
						FillRect( m_hdcBitmap, &rct, brDragging );
						}
					}

				//-- Draw text on background
				if( iHgt > 12 ) {
					rct=rctBase;
					SetRect( &rct, 0, iHgt*iTrace, iWid-1, iHgt*(iTrace+1)-1 );
					sprintf( ach, "%d", iSeg );	//, iTrace+m_iCurrentTrace 
					//ExtTextOut( m_hdcBitmap, 0, iHgt*iTrace, ETO_CLIPPED, &rct, ach, strlen(ach), NULL );
					DrawText( m_hdcBitmap, ach, -1, &rct, DT_BOTTOM|DT_RIGHT );
					}

				//-- ensure data for segment has been loaded and decimated
				if( pSeg == NULL && iSeg>=0 && iSeg<m_oData.m_aSegments.Count() ) { 
					pSeg = m_oData.Segment(iSeg);
					
					bool bSetRange = ( pSeg->m_pdData == NULL );
					
					//-- Load segment data 
					if( pSeg->m_pdData == NULL ) 
						m_oData.m_pDataFile->LoadSegmentData( pSeg );
					
					//-- set ranges
					if( bSetRange && pSeg->m_pdData != NULL ) 
						pSeg->SetRanges( &(m_oData.m_oConfig.m_aChannels) );

					//-- Decimate Data 
					pSeg->CreateDisp( iHgt-2, dPixPerSamp, m_oData.m_oConfig.m_bOverlap, 
									  pSeg->m_bDigout && m_oData.m_oConfig.m_bDispDigout, 
									  &(m_oData.m_oConfig.m_aChannels) );
					}
					
				//-- render to bitmap
				if( pSeg != NULL ) { 
					//-- render 1 trace 
					pSeg->Render( m_hdcBitmap, 
									m_oData.m_oConfig.m_bOverlap,	
									m_oData.m_oConfig.m_bDispIdealizations,
									dPixPerSamp,
									iHgt*iTrace, 
									MulDiv(iOffset,iWid,m_oData.m_oConfig.m_iPointsPerTrace), /* todo - is this exact enough ? */
									iWid, 
									&(m_oData.m_oConfig.m_aChannels), 
									m_oData.m_oConfig.m_acrDigout );
									
					//-- advance pointers
					iOffset+=m_oData.m_oConfig.m_iPointsPerTrace;
					if( iOffset >= pSeg->m_iSamples ) { 
						iOffset=0;
						pSeg=NULL;
						iSeg++;
						}
					}
				}

			//-- clean up 
			DeleteObject( brFill[0] );
			DeleteObject( brFill[1] );
			DeleteObject( brListHighlight );
			DeleteObject( brListItem );
			DeleteObject( brHighlight );
			SelectObject( m_hdcBitmap, penWas );
			DeleteObject( penRed );
			}
				
		void OnPaint() {
			//--- Don't continuously redraw while sizing
			if( m_bSizing ) 
				return;
				
			//----- If data to draw on bitmap has changed, update 
			if( ! m_bSetupBitmap ) {
				m_bDataDrawn=false;
				
				//-- Clear prior bitmap object
				if( m_hBitmap != NULL ) {
					DeleteObject( m_hBitmap );
					DeleteDC( m_hdcBitmap );
					}
					
				//-- Create bitmap for buffering display 
				HDC hdc = GetDC( m_hDlg );
				int iWid = m_rctData.right-m_rctData.left-1;
				int iHgt = m_rctData.bottom-m_rctData.top-2;
				m_hdcBitmap = CreateCompatibleDC( hdc );
				m_hBitmap = CreateCompatibleBitmap( hdc, iWid, iHgt );
				SelectObject( m_hdcBitmap, m_hBitmap );
				PatBlt( m_hdcBitmap, 0, 0, iWid, iHgt, WHITENESS );
				ReleaseDC( m_hDlg, hdc );

				m_bSetupBitmap=true;
				}

			//-- Draw data to off screen bitmap 
			if( ! m_bDataDrawn ) 
				DrawData();				

			//------------------------------------------------------ Update the display
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(m_hDlg, &ps);
			
			//----- Copy from bitmap to display
			BitBlt( hdc, 
					m_rctData.left+1, 
					m_rctData.top+1, 
					m_rctData.right-m_rctData.left-1, 
					m_rctData.bottom-m_rctData.top, 
					m_hdcBitmap, 0, 0, SRCCOPY );

			EndPaint(m_hDlg, &ps);
			}

		void OnMinMaxInfo( MINMAXINFO * p ) {  
			p->ptMinTrackSize.x = 690;
			p->ptMinTrackSize.y = 200;
			}

		void OnKeyDown( DWORD wParam, DWORD lParam ) {
			if( wParam==109 || wParam==107 ) {			// - or + 
				bool bCtl = ( GetKeyState( VK_CONTROL ) & 0x0080 ) > 0  ;
				int iRep = ( lParam & 0x00FF );
				while ( --iRep>=0 )
					if( bCtl ) 
						OnSpinTraces(wParam==109);
					else 
						OnSpinPoints(wParam==109);
				}
			}

		void OnKeyUp( DWORD wParam, DWORD lParam ) {  
			}

		void OnChar( DWORD wParam, DWORD lParam ) {  
			}

		void OnListSelChanged( bool bScrollToSelection ) { 
			if( bScrollToSelection ) { 
				//-- scroll view to include start of selected list item.
				int iList = m_cmbList.GetCurSel();
				int iCurrItem = m_cmbListItems.GetCurSel();
				CSelList * pList;
				CSelListItem * pItem;
				if( ( pList=m_oData.List(iList) ) != NULL 
						&& ( pItem=pList->Item(iCurrItem) ) != NULL ) 
					GoToPoint( pItem->m_iStartSegment, pItem->m_iStartPoint );
				}
		
			m_bDataDrawn=false;
			InvalidateRect( m_hDlg , NULL, TRUE );
			}

		void OnDataChanged() { 
			SetScrollbarRange();
			m_bDataDrawn=false;
			InvalidateRect( m_hDlg , NULL, TRUE );
			}
		
		//-----------------------------------------------------------------------
		/*
		BOOL OnKey( DWORD dwVKey, DWORD dwKey ) { 
			switch( dwKey ) { 
				case 37 :				// left
					break;
				case 39 :				// right
					break;
				case 38 :				// Up
					GoToTrace( m_iCurrentTrace-1 );
					break;
				case 40 :				// down
					GoToTrace( m_iCurrentTrace+1 );
					break;
				case 36 :				// Home (CW)
					break;
				case 35 :				// End (CCW)
					break;
				case 45 :				// - 
				case 189 :
					break;
				case 43 :				// + 
				case 187 :
					
					break;
					
				default : 
					#ifdef _DEBUG
						char buffer[100];
						sprintf( buffer, "ED Key : %d\n", int(dwKey) );
						OutputDebugString(  buffer );
					#endif
					return FALSE;
				}
						
			return TRUE;
			} 		
		*/
		
		void OnSize( WPARAM wParam, LPARAM * p ) { 
			if( wParam == SIZE_MINIMIZED ) 
				return;
				
			/*	In response to a resizing, which includes the initial sizing of the window, 
				this function moves the picture display and scroll bar controls.  A bitmap 
				of the same size as the picture control is created here also.
			*/
			
			//-- Desired size
			int iDlgWid=LOWORD(*p);
			int iDlgHgt=HIWORD(*p);

			//-- dialogue window info 
			RECT rcPar;
			GetBounds( &rcPar );
			MoveWindow( m_hDlg, 0, 0, iDlgWid, iDlgHgt, TRUE );
			
			//-- Resize picture control 
			m_picData.GetControlBounds( &m_rctData );
			m_rctData.left--;							
			m_rctData.top--;							
			m_rctData.right = iDlgWid-(rcPar.right-m_rctData.right)+1;		// indent right same as left
			m_rctData.bottom = iDlgHgt-(rcPar.bottom-m_rctData.bottom)+1;	// indent bottom same at left 
			MoveWindow( m_picData.GetHandle(), 
							m_rctData.left, 
							m_rctData.top, 
							m_rctData.right-m_rctData.left,
							m_rctData.bottom-m_rctData.top,	
							TRUE );

			//-- Resize scroll bar control 
			RECT rcScr;
			m_scrData.GetControlBounds( &rcScr );
			int iScrWid = rcScr.right-rcScr.left+2;							
			MoveWindow( m_scrData.GetHandle(), 
							m_rctData.right, 
							m_rctData.top, 
							iScrWid,
							m_rctData.bottom-m_rctData.top,	
							TRUE );
							
			//-- Move status control
			RECT rcSt;
			m_chkStatus.GetControlBounds( &rcSt );
			InflateRect( &rcSt,1,1 );
			MoveWindow( m_chkStatus.GetHandle(), 
							rcSt.left, 
							iDlgHgt-(rcSt.bottom-rcSt.top)-1, 
							iDlgWid-2*rcSt.left /*rcSt.right-rcSt.left*/, 
							rcSt.bottom-rcSt.top, 
							TRUE );

			//-- Move status clipboard button 
			m_btnStatusCopy.GetControlBounds( &rcSt );
			InflateRect( &rcSt,1,1 );
			MoveWindow( m_btnStatusCopy.GetHandle(), 
							rcSt.left, 
							iDlgHgt-(rcSt.bottom-rcSt.top)-1, 
							rcSt.right-rcSt.left, 
							rcSt.bottom-rcSt.top, 
							TRUE );


			//-- remember that bitmap has been setup
			m_bSetupBitmap=false;
			}			

		//---- Create a new data object 
		void CreateNewData( COptionsNewData * pConfig ) { 
			//-- Header
			m_oData.SetStatus(0);	//TODO - set to busy, then reset later 
			m_cFileName=pConfig->m_cFileName;	
					
			SetWindowText( GetParent(m_hDlg), m_cFileName.ConstBuffer() );
			m_oData.m_dSampling=pConfig->m_dSampling;
			m_oData.m_dScaling=pConfig->m_dScaling;
			
			//-- Create channels : Response + any stimulations
			m_oData.m_oConfig.m_aChannels.DeleteAll();
			for( int iCh=0; iCh<pConfig->m_iChannelCount; ++iCh ) 
				m_oData.m_oConfig.AddChannel();
			
			m_oData.m_oConfig.m_iPointsPerTrace=pConfig->m_iPointsPerTrace;
			m_oData.m_oConfig.m_iTracesPerPage=pConfig->m_iTracesPerPage;

			m_cmbList.ClearContents();
			m_cmbListItems.ClearContents();
			}

		//---- Create a new simulation data object 
		void CreateNewData( COptionsSimulate * pConfig, double dScaling=1.0) { 

			//-- Header
			m_oData.SetStatus(0);	//TODO - set to busy, then reset later 
			m_cFileName=pConfig->m_cFileName;	
					
			SetWindowText( GetParent(m_hDlg), m_cFileName.ConstBuffer());
			m_oData.m_dSampling=1.0/pConfig->m_iSampleRate;
			m_oData.m_dScaling=dScaling;
			//-- Create channels : Response + any stimulations
			m_oData.m_oConfig.m_aChannels.DeleteAll();
			m_oData.m_oConfig.AddChannel();
			
			m_oData.m_oConfig.m_iPointsPerTrace=max(1000,min(1000000,pConfig->m_iPointCount ) );
			m_oData.m_oConfig.m_iTracesPerPage=max(1,min(10,pConfig->m_iSegments ) );

			m_cmbList.ClearContents();
			m_cmbListItems.ClearContents();
			}
			
		void SetChannelUnits( int iChannel, const char * pchUnits ) { 
			m_oData.m_oConfig.Channel(iChannel)->m_cUnits.Set( pchUnits );
			}

		//---- Add a new data trace to a data object 
		CSegment * AddSegmentData( double * pData, int iChannels, int iSamples ) { 
			CSegment * pSeg = new CSegment;
			m_oData.m_aSegments.AddItem( pSeg );
			pSeg->m_dtStartTime = 0.0;
			pSeg->m_iPos=0;
			pSeg->m_pdData=NULL;
			pSeg->SetData( pData, iSamples, iChannels, true );
			pSeg->SetRanges( &m_oData.m_oConfig.m_aChannels );
			m_oData.m_bDataUpdated = true;
			return pSeg;
			}
				
		void AddSegmentDataAndIdl( double * pData, int iChannels, CIdlList * pIdlList ) { 
			//int iStatus = m_oData.GetStatus()
			//if( iStatus!=2 )
			//	m_oData.SetStatus(2);
			CSegment * pSeg = AddSegmentData( pData, iChannels, pIdlList->m_iLength );
			pSeg->m_aIdl.AddItem( pIdlList );
			m_oData.SetIdlUpdated();
			//if( iStatus!=2 )
			//	m_oData.SetStatus(iStatus);
			m_oData.m_bDataUpdated = true;
			}
			
		void OnLoadFile( const char * pchFileName ) { 
			int iLen=strlen(pchFileName);
			
			/*
			if( iLen>5 && stricmp( pchFileName+iLen-4, ".abf" ) == 0 ) { 
				if( IDYES==MessageBox( m_hDlg, "Convert from ABF format to QDF format ?", "Open ABF file", MB_YESNO ) ) {
			// TODO - link to conversion utility 		
					char achNewFileName[_MAX_PATH];
					strcpy( achNewFileName, pchFileName );
					strcpy( achNewFileName+iLen-4, ".qdf" );
					int iErr = ABFConvert( pchFileName, achNewFileName );
					if( iErr == 0 ) {
						OnLoadFile( achNewFileName );
						return;
						}
					}
				return;
				}
			*/
							
			if( m_oData.OpenQDF( pchFileName ) ) {
				SetWindowText( GetParent(m_hDlg), pchFileName );
				m_bDataDrawn=false;
				FillListControl( m_oData.m_aLists.Count()-1 );
				FillListItemControl();
				m_cFileName=pchFileName;
				OnDataChanged(); 
				}
			else {
				MessageBox( GetParent(m_hDlg), "Error opening data file", pchFileName, MB_OK );
				m_oData.ClearUpdated();
				DataMenuOff();
				DestroyWindow( GetParent( m_hDlg ) );
				}
			}

		void OnScrollData( WPARAM wp ) { 
			int iPos=m_scrData.ScrollProc( wp );
			if( iPos != -1 ) 
				GoToTrace( iPos ) ;
			}

		void SetScrollbarRange() { 
			int iTraces=0;
			for( int iSeg=0; iSeg<m_oData.m_aSegments.Count(); ++iSeg )
				iTraces += ((CSegment *) m_oData.Segment(iSeg))->TraceCount( m_oData.m_oConfig.m_iPointsPerTrace );
			iTraces=max(0,iTraces-m_oData.m_oConfig.m_iTracesPerPage);
			m_scrData.SetRange( 0, iTraces, max(1,iTraces/100), max(2,iTraces/10) );
			}
			
		void OnSpinPoints( bool bIncrement ) {
			SendMessage( m_spinPoints.GetHandle(), UDM_SETPOS, 0, SPINMID);

			if( bIncrement ) {
				int iIncr=int(pow(10,int(log10(double(m_oData.m_oConfig.m_iPointsPerTrace)))));
				m_oData.m_oConfig.m_iPointsPerTrace=min( m_oData.m_oConfig.m_iPointsPerTrace+iIncr, MAX_POINTS );
				}
			else {
				int iIncr=int(pow(10,int(log10(double(m_oData.m_oConfig.m_iPointsPerTrace))-0.25)));
				m_oData.m_oConfig.m_iPointsPerTrace=max( 100, m_oData.m_oConfig.m_iPointsPerTrace-iIncr );
				}
			OnDataChanged();
			m_oData.SetConfigUpdated();
			}

		void OnSpinTraces( bool bIncrement ) {
			SendMessage( m_spinTraces.GetHandle(), UDM_SETPOS, 0, SPINMID);
			if( bIncrement ) 
				m_oData.m_oConfig.m_iTracesPerPage=max( 1, m_oData.m_oConfig.m_iTracesPerPage-1 );
			else
				m_oData.m_oConfig.m_iTracesPerPage=min( m_oData.m_oConfig.m_iTracesPerPage+1, MAX_TRACES );
			OnDataChanged();
			m_oData.SetConfigUpdated();
			}

		void GoToTrace( int iTrace ) { 
			if( iTrace==0 ) { 
				m_iDrawSeg=0;
				m_iDrawOffset=0;
				m_iCurrentTrace=0;
				}
			else { 		
				CSegment * pSeg;
				int iTraces, iPtsPerTrace = m_oData.m_oConfig.m_iPointsPerTrace;
				//-- backward through traces 
				while( iTrace < m_iCurrentTrace ) { 
					if( m_iDrawOffset > 0 ) {
						//-- backwards within trace
						iTraces = min( int((m_iDrawOffset+iPtsPerTrace-1)/iPtsPerTrace), m_iCurrentTrace-iTrace );
						m_iDrawOffset = max(m_iDrawOffset-iPtsPerTrace*iTraces,0);
						m_iCurrentTrace -= iTraces;
						}
					else if( m_iDrawSeg > 0 ) { 
						//-- backwards to prior trace
						pSeg = (CSegment *) m_oData.Segment(--m_iDrawSeg);
						iTraces = int( (pSeg->m_iSamples-1) / iPtsPerTrace )+1;
						m_iDrawOffset = max(0,iTraces-(m_iCurrentTrace-iTrace)) * iPtsPerTrace;
						m_iCurrentTrace = max( m_iCurrentTrace-iTraces, iTrace );							
						}
					else {
						iTrace = m_iCurrentTrace;
						}
					}

				//-- forward through traces 
				while( iTrace > m_iCurrentTrace ) { 
					pSeg = (CSegment *) m_oData.Segment(m_iDrawSeg);
					if( m_iDrawOffset + (iTrace-m_iCurrentTrace)*iPtsPerTrace > pSeg->m_iSamples ) { 
						//-- forward to next trace 
						if( m_iDrawSeg+1 < m_oData.m_aSegments.Count() ) {
							iTraces = (pSeg->m_iSamples-m_iDrawOffset-1)/iPtsPerTrace+1;
							m_iCurrentTrace+=iTraces;
							m_iDrawOffset = 0;
							++m_iDrawSeg;
							}
						else { // at end - no more segments
							iTrace=m_iCurrentTrace;
							}
						}
					else { 
						//-- forward within trace 
						iTraces = iTrace-m_iCurrentTrace;
						m_iDrawOffset = min(m_iDrawOffset+iPtsPerTrace*iTraces,pSeg->m_iSamples-1);
						m_iCurrentTrace += iTraces;
						}
					}					
				}			
			m_bDataDrawn=false;
			InvalidateRect( m_hDlg , NULL, FALSE );
			}
		
		void GoToPoint( int iSegment, int iPoint ) { 
			int iPtsPerTrace = m_oData.m_oConfig.m_iPointsPerTrace;
			CSegment * pSeg;
			if( iPtsPerTrace==0 
				|| (pSeg=m_oData.Segment(iSegment))==NULL
				|| pSeg->m_iSamples<iPoint ) 
				return;

			m_iDrawSeg=0;
			m_iDrawOffset=0;
			m_iCurrentTrace=0;
			while( m_iDrawSeg<iSegment ) { 
				pSeg = m_oData.Segment(m_iDrawSeg);
				m_iCurrentTrace += (pSeg->m_iSamples-1)/iPtsPerTrace+1;
				++m_iDrawSeg;
				}
			if( m_oData.m_oConfig.m_iTracesPerPage*iPtsPerTrace < iPoint ) { 
				int iTraces=(iPoint-1)/iPtsPerTrace;
				m_iCurrentTrace += iTraces;
				m_iDrawOffset += iTraces*iPtsPerTrace;
				}
				
			m_scrData.SetPos(m_iCurrentTrace);
			m_bDataDrawn=false;
			InvalidateRect( m_hDlg , NULL, FALSE );
			}
			
		//--------------------------------------- Mouse handler
		bool DragDown( short iX, short iY ) { 			
			POINT p;
			p.x=iX;
			p.y=iY;
			//----- Click in Disp area  
			if( PtInRect( &m_rctData, p ) ) { 
				SetCapture( m_hDlg );
				m_bDrag=true;
				m_iXDrag=m_iXHit=iX;
				m_iYDrag=m_iYHit=iY;
				m_bDataDrawn=true;
				InvalidateRect( m_hDlg, &m_rctData, FALSE ); 
				return true;
				}
			return false;
			}

		void DragMove( short iX, short iY ) { 			
			m_iXDrag=iX;
			m_iYDrag=iY;
			m_bDataDrawn=false;
			InvalidateRect( m_hDlg, &m_rctData, FALSE ); 
			}

		void DragUp( short iX, short iY ) { 			
			ReleaseCapture();
			InvalidateRect( m_hDlg, &m_rctData, FALSE ); 
			m_bDataDrawn=false;
			m_bDrag=false;
			
			// determine what was selected 
			PointToSegmentOffset( m_iXHit, m_iYHit, &m_iSelStartSeg, &m_iSelStartOffset );
			PointToSegmentOffset( iX, iY, &m_iSelStopSeg, &m_iSelStopOffset );
			if( m_iSelStartSeg>m_iSelStopSeg || ( m_iSelStartSeg==m_iSelStopSeg && m_iSelStartOffset>m_iSelStopOffset ) ) { 
				// stop < start : swap start and stop 
				m_iSelStartSeg=m_iSelStopSeg;
				m_iSelStartOffset=m_iSelStopOffset;
				PointToSegmentOffset( m_iXHit, m_iYHit, &m_iSelStopSeg, &m_iSelStopOffset );
				}
			SetStatusText();
			m_bSelect=true;
			}
			
		void DragDoubleClick( int iX, int iY ) { 			
			m_iXHit=0;
			m_iYHit=iY;
			m_iXDrag=m_rctData.right;
			m_iYDrag=iY;
			
			// Note : For double click to select left trace to cursor instead of full trace, Change m_iXDrag to iX 
			DragUp( m_iXDrag, iY );	
			}

		//-----Menu Option Handlers 				
		void OnMDIActivate( HWND hwndLosing, HWND hwndGaining ) {
			if( GetParent(m_hDlg)==hwndLosing ) {
				DataMenuOff();
				}
			if( GetParent(m_hDlg)==hwndGaining ) {
				DataMenuOn();
				}
			}
			
		void OnCursorNew() { 
			//TODO
			}
		void OnCursorDelete() { 
			//TODO
			}
		void OnCursorDeleteAll() { 
			//TODO
			}
			
		void OnListNew() { 
			CString cNewList="List";
			if( GetSingleEntry( m_hDlg, "New List", "List Name:", &cNewList ) ) {
				//-- ensure name is not empty
				if( cNewList.RTrim().Length()==0 ) { 
					MessageBeep(0);
					return;
					}
				//-- ensure name is not a duplicate
				if( m_oData.List( cNewList.ConstBuffer() ) != NULL ) {
					MessageBox( m_hDlg, "A list with that name already exists.","New List", MB_OK);
					return;
					}
				//-- add to data 
				m_oData.m_aLists.AddItem( new CSelList( cNewList.Buffer() ) );
				//-- add to combo controls
				FillListControl( m_oData.m_aLists.Count()-1 );
				FillListItemControl();
				//-- set updated flag 
				m_oData.SetListsUpdated();
				OnListSelChanged( false );
				}
			}
			
		void OnListDelete() { 
			int iSel = m_cmbList.GetCurSel();
			if( iSel<0 )
				MessageBeep(0);
			else if( IDYES==MessageBox( m_hDlg, "Delete the selected list ?", "Confirm delete", MB_YESNO ) )  { 
				//-- remove from data 
				m_oData.m_aLists.DeleteItem( iSel );
				//-- remove from combo controls
				FillListControl( m_oData.m_aLists.Count()-1 );
				FillListItemControl();
				//-- set updated flag 
				m_oData.SetListsUpdated();
				OnListSelChanged( true );
				}
			}
			
		void OnListDeleteAll() { 
			int iSel = m_cmbList.GetCurSel();
			if( iSel<0 )
				MessageBeep(0);
			else if( IDYES==MessageBox( m_hDlg, "Delete entire list contents for the selected list ?", "Confirm delete", MB_YESNO ) ) {
				m_oData.List( iSel )->m_aItems.DeleteAll();
				FillListItemControl();
				m_oData.SetListsUpdated();
				OnListSelChanged( false );
				}
			}

		void OnListAddAllSegments() { 
			CString cNewList="All Segs";
			
			if( GetSingleEntry( m_hDlg, "New All Segments List", "List Name:", &cNewList ) ) {
				//-- ensure name is not empty
				if( cNewList.RTrim().Length()==0 ) { 
					MessageBeep(0);
					return;
					}
				//-- ensure name is not a duplicate
				if( m_oData.List( cNewList.ConstBuffer() ) != NULL ) {
					MessageBox( m_hDlg, "A list with that name already exists.","New List", MB_OK);
					return;
					}
			
				//-- add to data 
				CSelList * pList = new CSelList( cNewList.Buffer() );
				m_oData.m_aLists.AddItem( pList );
				for( int iSeg=0; iSeg<m_oData.m_aSegments.Count(); ++iSeg ) { 
					CSegment * pSeg = m_oData.Segment(iSeg);
					CSelListItem * pItem = new CSelListItem();
					pItem->Set( iSeg, 0, iSeg, pSeg->m_iSamples-1 );
					pList->m_aItems.AddItem( pItem );
					}
				//-- add to combo controls
				FillListControl( m_oData.m_aLists.Count()-1 );
				FillListItemControl( 0 );
				//-- set updated flag 
				m_oData.SetListsUpdated();
				OnListSelChanged( true );
				}
			}

		void OnListNewItem() { 
			//-- get current list.  Exit if none, or no data selected.
			int iSel = m_cmbList.GetCurSel();
			if( iSel<0 || (m_iSelStartSeg==m_iSelStopSeg && m_iSelStartOffset==m_iSelStopOffset) ) {
				MessageBeep(0);
				return;
				}
			CSelList * pList = m_oData.List(iSel);
			
			//-- add new item to list
			int iPos = pList->AddListItem( m_iSelStartSeg, m_iSelStartOffset, m_iSelStopSeg, m_iSelStopOffset );
			CSelListItem * pItem = pList->Item(iPos);
			
			//-- update control contents 
			char buf[32];
			pItem->Text(buf);
			m_cmbListItems.InsertString( buf, iPos );
			m_cmbListItems.SetCurSel( iPos );

			//-- set updated flag 
			m_oData.SetListsUpdated();
			OnListSelChanged( false );
			}

		void OnListItemInfo() { 
			//-- locate selected item
			int iSel = m_cmbList.GetCurSel();
			if( iSel<0 || iSel>=m_oData.m_aLists.Count() )
				return;
			CSelList * pList = m_oData.List( iSel );

			int iItem = m_cmbListItems.GetCurSel();
			if( iItem<0||iItem>=pList->m_aItems.Count() )
				return;
			CSelListItem * pItem = pList->Item(iItem);
			
			//-- display / modify label and note
			if( EditListItem( m_hDlg, &(pItem->m_cLabel), &(pItem->m_cNote) ) ) {
				FillListItemControl( m_cmbListItems.GetCurSel() );
				m_oData.SetListsUpdated();
				OnListSelChanged( false );
				}
			}
			
		void OnListDeleteItem() { 
			int iSel = m_cmbList.GetCurSel();
			int iItem = m_cmbListItems.GetCurSel();
			if( iSel<0 || iItem<0 )
				return;

			if( IDYES==MessageBox( m_hDlg, "Delete selected list item ?", "Confirm delete", MB_YESNO ) ) {
				CSelList * pList = m_oData.List( iSel );
				pList->m_aItems.DeleteItem( iItem );
				FillListItemControl( max(iItem,pList->m_aItems.Count())-1 );
				m_oData.SetListsUpdated();
				OnListSelChanged( true );
				}
			}

		void OnListSelect() {		// a new list has been highlighted in the dropdown
			FillListItemControl( 0 );
			OnListSelChanged( true );
			} 
			
		void OnListItemSelect() {		// a new listitem has been selected
			//-- cause redraw for highlighting 
			OnListSelChanged( true );
			}

		void OnFitcurveNew() { 
			//TODO
			}
		void OnFitcurveDelete() { 
			//TODO
			}
		void OnFitcurveDeleteAll() { 
			//TODO
			}

		void OnIdlNew() { 
			int iList = m_cmbList.GetCurSel();
			CString cList;
			m_cmbList.GetText( & cList );
			OpenVwIdealize( m_hDlg, &m_oData, 
						false, 
						iList>=0?cList.ConstBuffer():NULL, /* (current list?) and (list name) */
						m_bSelect, m_iSelStartSeg, m_iSelStartOffset, m_iSelStopSeg, m_iSelStopOffset );
			}
			
		void OnIdlDelete() { 
			//TODO
			}
			
		void OnIdlDeleteAll() { 
			//TODO
			}
			
		void OnAnalyzeMil() { 
			//TODO
			}
			
		void OnAmpHistogram() { 
			int iList = m_cmbList.GetCurSel();
			CString cList;
			m_cmbList.GetText( & cList );
			OpenVwAmpHistogram( m_hDlg, &m_oData, 
						iList>=0, iList>=0?cList.ConstBuffer():NULL, /* (current list?) and (list name) */
						m_bSelect, m_iSelStartSeg, m_iSelStartOffset, m_iSelStopSeg, m_iSelStopOffset );
			}
			
		void OnExportAscii() { 
			int iList = m_cmbList.GetCurSel();
			CString cList;
			m_cmbList.GetText( & cList );
			OpenVwExport( m_hDlg, &m_oData, iList>=0, m_bSelect );
			}
			
		void OnExportIdealization() { 
			//TODO
			}
			
		};


//-----------------------------------------------------------------------------
BOOL CALLBACK DataviewDlgProc(HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam) { 
	CVwData * pVw;
	
	//-- create the CVwData to go with the dialog 
	if( iMsg == WM_INITDIALOG ) { 
		pVw = new CVwData;
		pVw->m_hDlg=hDlg;
		pVw->InitializeControls();
		pVw->OnMDIActivate( NULL, GetParent(hDlg) );
		SetWindowLong( GetParent(hDlg), 0, (long) pVw ) ;
		return TRUE;
		}
	
	// Get the CVwData pointer for this dialog 
	pVw = (CVwData *) GetWindowLong(  GetParent( hDlg ), 0 );

	int wmId, wmEvent;

	switch( iMsg ) { 
		case WM_LBUTTONDBLCLK :		//-- 
			pVw->DragDoubleClick( LOWORD(lParam), HIWORD(lParam) );
			return TRUE;
			
		case WM_LBUTTONDOWN :		//-- Start of drag operation			
			if( pVw->DragDown( LOWORD(lParam), HIWORD(lParam) ) ) 
				return TRUE;
			return FALSE;

		case WM_MOUSEMOVE :			//-- Mouse move during drag operation
			if( ! pVw->m_bDrag)
				return FALSE;
			pVw->DragMove( LOWORD(lParam), HIWORD(lParam) );
			return TRUE;

		case WM_LBUTTONUP :			//-- end of drag operation
			if( ! pVw->m_bDrag )
				return FALSE;
			pVw->DragUp( LOWORD(lParam), HIWORD(lParam) );
			return TRUE;

		/*
		case WM_CTLCOLORSTATIC :
			wmId = GetDlgCtrlID( (HWND)lParam );
			if( wmId >= ID_GENCTL && wmId < ID_GENCTL+5000 ) 
				return (BOOL)GetStockObject( WHITE_BRUSH );
			return FALSE;
		*/

		case WM_NOTIFY :		// spin controls
			wmId = LOWORD(wParam); 
			
			//----- Change PointsPerTrace
			if( wmId == IDC_SPINPOINTS ) {
				pVw->OnSpinPoints( ((LPNMUPDOWN) lParam)->iDelta < 0 ) ;
				return FALSE;
				}

			if( wmId == IDC_SPINTRACES ) {
				pVw->OnSpinTraces( ((LPNMUPDOWN) lParam)->iDelta < 0 ) ;
				return FALSE;
				}
			break;

		case WM_DATA_LOADFILE :
			pVw->OnLoadFile( (char *) lParam );
			return TRUE;
			
		case WM_DATA_CREATENEW :
			pVw->CreateNewData( (COptionsNewData *) wParam );
			return true;

		case WM_DATA_CHANNELINFO :
			pVw->SetChannelUnits( (int) wParam, (const char *)lParam );
			return true;

		case WM_DATA_CREATESIM :
			pVw->CreateNewData( (COptionsSimulate *) wParam );
			return true;

		case WM_DATA_CHANGED :
			pVw->OnDataChanged();
			if( wParam ) 
				PostMessage( GetClientWin(), WM_MDIACTIVATE, (WPARAM)GetParent(pVw->m_hDlg), 0 );
			return true;
			

		case WM_DATA_ADDSEGMENT :
			pVw->AddSegmentData( (double *)lParam, pVw->m_oData.m_oConfig.ChannelCount(), (int)wParam );
			return true;
				
		case WM_DATA_ADDIDLSEG : 
			pVw->AddSegmentDataAndIdl( (double *)lParam, 1, (CIdlList *)wParam );
			return true;

		case WM_DATA_LOADED :
			//TODO
			return TRUE;
		
		case WM_PAINT :
			pVw->OnPaint();
			return FALSE;

		case WM_COMMAND :
			wmId = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			switch( wmId ) { 
				//-- Cursor message handlers
				case ID_DATACURSOR_NEW :
					pVw->OnCursorNew();
					break;
				case ID_DATACURSOR_DELETE :
					pVw->OnCursorDelete();
					break;
				case ID_DATACURSOR_DELETEALL :
					pVw->OnCursorDeleteAll();
					break;
					
				//-- Selection List message handlers
				case IDC_CMBLIST :
					if( wmEvent==CBN_SELCHANGE )
						pVw->OnListSelect();
					break;
				case IDC_CMBLISTITEM : 
					if( wmEvent==CBN_SELCHANGE )
						pVw->OnListItemSelect();
					break;
					
				case IDC_BTNADDLIST : 
				case ID_DATALIST_NEW :
					pVw->OnListNew();
					break;
				case IDC_BTNDELLIST : 
				case ID_DATALIST_DELETE :
					pVw->OnListDelete();
					break;
				case IDC_BTNADDLISTITEM : 
				case ID_DATALIST_NEWITEM :
					pVw->OnListNewItem();
					break;
				case IDC_BTNDELLISTITEM : 
				case ID_DATALIST_DELETEITEM :
					pVw->OnListDeleteItem();
					break;
				case IDC_BTNLISTITEMINFO : 
					pVw->OnListItemInfo();
					break;
				case IDC_BTNADDALLSEGMENTS :
				case ID_DATALIST_ADDALLSEGMENTS :
					pVw->OnListAddAllSegments();
					break;
				case ID_DATALIST_DELETEALL :
					pVw->OnListDeleteAll();
					break;
					
				//-- Fit curves - Menu handlers 
				case ID_DATAFITCURVE_NEW :
					pVw->OnFitcurveNew();
					break;
				case ID_DATAFITCURVE_DELETE :
					pVw->OnFitcurveDelete();
					break;
				case ID_DATAFITCURVE_DELETEALL :
					pVw->OnFitcurveDeleteAll();
					break;
				
				//-- IDL menu handlers
				case IDC_BTNIDL :
				case ID_DATAIDL_NEW :
					pVw->OnIdlNew();
					break;
				case ID_DATAIDL_DELETE :
					pVw->OnIdlDelete();
					break;
				case ID_DATAIDL_DELETEALL :
					pVw->OnIdlDeleteAll();
					break;
					
				//-- Misc handlers
				case IDC_CHK_STATUS : 
					pVw->OnCheckStatus();
					break;
				case ID_DATAANALYZE_MIL :
					pVw->OnAnalyzeMil();
					break;
				case ID_ANALYZE_AMPHISTOGRAM : 
					pVw->OnAmpHistogram();
					break;
				case ID_DATAEXPORT_ASCII :
					pVw->OnExportAscii();
					break;
				case ID_DATAEXPORT_IDEALIZATION :
					pVw->OnExportIdealization();
					break;			
				case IDC_BTNCONFIG :
					pVw->OnBtnConfig();
					break;
				case IDC_BTNSTATUSCOPY :
					pVw->CopyStatusToClipboard();
					break;
				
				default : 
					return FALSE;
				}
			return TRUE;

		case WM_DESTROY :
			pVw->OnExit();	
			delete pVw;
			SetWindowLong( GetParent(hDlg), 0, 0 ) ;
			WindowListRemove( hDlg );
			return TRUE;
			
		case WM_VSCROLL : 
			if( ((HWND)lParam) == pVw->m_scrData.GetHandle() )
				pVw->OnScrollData( wParam );
				
			return TRUE;

		default : 
			break;
		}
		
	return FALSE;
	} 

